@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL.c,v 1.5.2.12 2000/09/07 07:52:14 hercules Exp $";
+ "@(#) $Id: SDL.c,v 1.5.2.14 2001/02/10 07:20:03 hercules Exp $";
 #endif
 
 /* Initialization code for SDL */
@@ -220,6 +220,9 @@ void SDL_Quit(void)
 							surfaces_allocated);
 	}
 #endif
+
+	/* Uninstall any parachute signal handlers */
+	SDL_UninstallParachute();
 }
 
 /* Return the library version number */
