@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,13 +22,14 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_mouse.c,v 1.4 2000/02/03 19:33:37 hercules Exp $";
+ "@(#) $Id: SDL_mouse.c,v 1.3.2.10 2001/02/10 07:20:04 hercules Exp $";
 #endif
 
 /* General mouse handling code for SDL */
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include "SDL_events.h"
 #include "SDL_events_c.h"
@@ -39,6 +40,8 @@ static char rcsid =
 /* These are static for our mouse handling code */
 static Sint16 SDL_MouseX = 0;
 static Sint16 SDL_MouseY = 0;
+static Sint16 SDL_DeltaX = 0;
+static Sint16 SDL_DeltaY = 0;
 static Uint8  SDL_ButtonState = 0;
 
 
@@ -48,6 +51,8 @@ int SDL_MouseInit(void)
 	/* The mouse is at (0,0) */
 	SDL_MouseX = 0;
 	SDL_MouseY = 0;
+	SDL_DeltaX = 0;
+	SDL_DeltaY = 0;
 	SDL_ButtonState = 0;
 
 	/* That's it! */
@@ -63,6 +68,17 @@ Uint8 SDL_GetMouseState (int *x, int *y)
 	return(SDL_ButtonState);
 }
 
+Uint8 SDL_GetRelativeMouseState (int *x, int *y)
+{
+	if ( x )
+		*x = SDL_DeltaX;
+	if ( y )
+		*y = SDL_DeltaY;
+	SDL_DeltaX = 0;
+	SDL_DeltaY = 0;
+	return(SDL_ButtonState);
+}
+
 static void ClipOffset(Sint16 *x, Sint16 *y)
 {
 	/* This clips absolute mouse coordinates when the apparent
@@ -93,17 +109,15 @@ int SDL_PrivateMouseMotion(Uint8 buttonstate, int relative, Sint16 x, Sint16 y)
 		buttonstate = SDL_ButtonState;
 	}
 
+	Xrel = x;
+	Yrel = y;
 	if ( relative ) {
 		/* Push the cursor around */
-		Xrel = x;
-		Yrel = y;
 		x = (SDL_MouseX+x);
 		y = (SDL_MouseY+y);
 	} else {
 		/* Do we need to clip {x,y} ? */
 		ClipOffset(&x, &y);
-		Xrel = x-SDL_MouseX;
-		Yrel = y-SDL_MouseY;
 	}
 
 	/* Mouse coordinates range from 0 - width-1 and 0 - height-1 */
@@ -123,9 +137,28 @@ int SDL_PrivateMouseMotion(Uint8 buttonstate, int relative, Sint16 x, Sint16 y)
 	else
 		Y = (Uint16)y;
 
+	/* If not relative mode, generate relative motion from clamped X/Y.
+	   This prevents lots of extraneous large delta relative motion when
+	   the screen is windowed mode and the mouse is outside the window.
+	*/
+	if ( ! relative ) {
+		Xrel = X-SDL_MouseX;
+		Yrel = Y-SDL_MouseY;
+	}
+
+	/* Update internal mouse state */
+	SDL_ButtonState = buttonstate;
+	SDL_MouseX = X;
+	SDL_MouseY = Y;
+	SDL_DeltaX += Xrel;
+	SDL_DeltaY += Yrel;
+	SDL_MoveCursor(SDL_MouseX, SDL_MouseY);
+
+	/* Post the event, if desired */
 	posted = 0;
 	if ( SDL_ProcessEvents[SDL_MOUSEMOTION] == SDL_ENABLE ) {
 		SDL_Event event;
+		memset(&event, 0, sizeof(event));
 		event.type = SDL_MOUSEMOTION;
 		event.motion.state = buttonstate;
 		event.motion.x = X;
@@ -137,13 +170,9 @@ int SDL_PrivateMouseMotion(Uint8 buttonstate, int relative, Sint16 x, Sint16 y)
 			SDL_PushEvent(&event);
 		}
 	}
-	/* Update internal mouse state */
-	SDL_ButtonState = buttonstate;
-	SDL_MouseX = X;
-	SDL_MouseY = Y;
-	SDL_MoveCursor(SDL_MouseX, SDL_MouseY);
 	return(posted);
 }
+
 int SDL_PrivateMouseButton(Uint8 state, Uint8 button, Sint16 x, Sint16 y)
 {
 	SDL_Event event;
@@ -151,6 +180,8 @@ int SDL_PrivateMouseButton(Uint8 state, Uint8 button, Sint16 x, Sint16 y)
 	int move_mouse;
 	Uint8 buttonstate;
 
+	memset(&event, 0, sizeof(event));
+
 	/* Check parameters */
 	if ( x || y ) {
 		ClipOffset(&x, &y);
@@ -191,6 +222,14 @@ int SDL_PrivateMouseButton(Uint8 state, Uint8 button, Sint16 x, Sint16 y)
 			return(0);
 	}
 
+	/* Update internal mouse state */
+	SDL_ButtonState = buttonstate;
+	if ( move_mouse ) {
+		SDL_MouseX = x;
+		SDL_MouseY = y;
+		SDL_MoveCursor(SDL_MouseX, SDL_MouseY);
+	}
+
 	/* Post the event, if desired */
 	posted = 0;
 	if ( SDL_ProcessEvents[event.type] == SDL_ENABLE ) {
@@ -203,14 +242,6 @@ int SDL_PrivateMouseButton(Uint8 state, Uint8 button, Sint16 x, Sint16 y)
 			SDL_PushEvent(&event);
 		}
 	}
-
-	/* Update internal mouse state */
-	SDL_ButtonState = buttonstate;
-	if ( move_mouse ) {
-		SDL_MouseX = x;
-		SDL_MouseY = y;
-		SDL_MoveCursor(SDL_MouseX, SDL_MouseY);
-	}
 	return(posted);
 }
 
