@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_blit.h,v 1.3.2.13 2000/08/31 20:09:53 hercules Exp $";
+ "@(#) $Id: SDL_blit.h,v 1.3.2.15 2001/02/13 10:05:50 hercules Exp $";
 #endif
 
 #ifndef _SDL_blit_h
@@ -418,7 +418,7 @@ do {						\
 /* Don't use Duff's device to unroll loops */
 #define DUFFS_LOOP(pixel_copy_increment, width)				\
 { int n;								\
-	for ( n=width; n > 0; --n ) {				\
+	for ( n=width; n > 0; --n ) {					\
 		pixel_copy_increment;					\
 	}								\
 }
@@ -429,4 +429,9 @@ do {						\
 
 #endif /* USE_DUFFS_LOOP */
 
+/* Prevent Visual C++ 6.0 from printing out stupid warnings */
+#if defined(_MSC_VER) && (_MSC_VER >= 600)
+#pragma warning(disable: 4550)
+#endif
+
 #endif /* _SDL_blit_h */
