@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_gamma.c,v 1.1.2.9 2000/10/23 04:40:56 hercules Exp $";
+ "@(#) $Id: SDL_gamma.c,v 1.1.2.12 2001/02/10 07:20:05 hercules Exp $";
 #endif
 
 /* Gamma correction support */
@@ -38,7 +38,7 @@ static char rcsid =
 #include "SDL_sysvideo.h"
 
 #ifdef USE_MATH_H
-static void CalculateGammaRamp(float gamma, Uint8 *ramp)
+static void CalculateGammaRamp(float gamma, Uint16 *ramp)
 {
 	int i;
 
@@ -52,7 +52,7 @@ static void CalculateGammaRamp(float gamma, Uint8 *ramp)
 	/* 1.0 gamma is identity */
 	if ( gamma == 1.0 ) {
 		for ( i=0; i<256; ++i ) {
-			ramp[i] = i;
+			ramp[i] = (i << 8) | i;
 		}
 		return;
 	} else
@@ -60,15 +60,15 @@ static void CalculateGammaRamp(float gamma, Uint8 *ramp)
 	{ int value;
 		gamma = 1.0f / gamma;
 		for ( i=0; i<256; ++i ) {
-			value = (int)(pow((double)i/256.0, gamma)*256.0 + 0.5);
-			if ( value > 255 ) {
-				value = 255;
+			value = (int)(pow((double)i/256.0, gamma)*65535.0+0.5);
+			if ( value > 65535 ) {
+				value = 65535;
 			}
-			ramp[i] = (Uint8)value;
+			ramp[i] = (Uint16)value;
 		}
 	}
 }
-static void CalculateGammaFromRamp(float *gamma, Uint8 *ramp)
+static void CalculateGammaFromRamp(float *gamma, Uint16 *ramp)
 {
 	/* The following is adapted from a post by Garrett Bass on OpenGL
 	   Gamedev list, March 4, 2000.
@@ -78,9 +78,9 @@ static void CalculateGammaFromRamp(float *gamma, Uint8 *ramp)
 
 	*gamma = 1.0;
 	for ( i = 1; i < 256; ++i ) {
-	    if ( (ramp[i] != 0) && (ramp[i] != 255) ) {
-	        double B = (i % 256) / 256.0;
-	        double A = ramp[i] / 255.0;
+	    if ( (ramp[i] != 0) && (ramp[i] != 65535) ) {
+	        double B = (double)i / 256.0;
+	        double A = ramp[i] / 65535.0;
 	        sum += (float) ( log(A) / log(B) );
 	        count++;
 	    }
@@ -101,7 +101,7 @@ int SDL_SetGamma(float red, float green, float blue)
 #ifdef USE_MATH_H
 	/* Prefer using SetGammaRamp(), as it's more flexible */
 	{
-		Uint8 ramp[3][256];
+		Uint16 ramp[3][256];
 
 		CalculateGammaRamp(red, ramp[0]);
 		CalculateGammaRamp(green, ramp[1]);
@@ -131,7 +131,7 @@ int SDL_GetGamma(float *red, float *green, float *blue)
 #ifdef USE_MATH_H
 	/* Prefer using GetGammaRamp(), as it's more flexible */
 	{
-		Uint8 ramp[3][256];
+		Uint16 ramp[3][256];
 
 		succeeded = SDL_GetGammaRamp(ramp[0], ramp[1], ramp[2]);
 		if ( succeeded >= 0 ) {
@@ -150,13 +150,12 @@ int SDL_GetGamma(float *red, float *green, float *blue)
 	return succeeded;
 }
 
-int SDL_SetGammaRamp(Uint8 *red, Uint8 *green, Uint8 *blue)
+int SDL_SetGammaRamp(Uint16 *red, Uint16 *green, Uint16 *blue)
 {
 	int succeeded;
 	SDL_VideoDevice *video = current_video;
 	SDL_VideoDevice *this  = current_video;	
 	SDL_Surface *screen = SDL_PublicSurface;
-	Uint8 *r, *g, *b;
 
 	/* Verify the screen parameter */
 	if ( !screen ) {
@@ -170,17 +169,14 @@ int SDL_SetGammaRamp(Uint8 *red, Uint8 *green, Uint8 *blue)
 	}
 
 	/* Fill the gamma table with the new values */
-	r = &video->gamma[0*256];
 	if ( red ) {
-		memcpy(r, red, 256*sizeof(*r));
+		memcpy(&video->gamma[0*256], red, 256*sizeof(*video->gamma));
 	}
-	g = &video->gamma[1*256];
 	if ( green ) {
-		memcpy(g, green, 256*sizeof(*r));
+		memcpy(&video->gamma[1*256], green, 256*sizeof(*video->gamma));
 	}
-	b = &video->gamma[2*256];
 	if ( blue ) {
-		memcpy(b, blue, 256*sizeof(*r));
+		memcpy(&video->gamma[2*256], blue, 256*sizeof(*video->gamma));
 	}
 
 	/* Gamma correction always possible on split palettes */
@@ -199,14 +195,14 @@ int SDL_SetGammaRamp(Uint8 *red, Uint8 *green, Uint8 *blue)
 	/* Try to set the gamma ramp in the driver */
 	succeeded = -1;
 	if ( video->SetGammaRamp ) {
-		succeeded = video->SetGammaRamp(this, r, g, b);
+		succeeded = video->SetGammaRamp(this, video->gamma);
 	} else {
 		SDL_SetError("Gamma ramp manipulation not supported");
 	}
 	return succeeded;
 }
 
-int SDL_GetGammaRamp(Uint8 *red, Uint8 *green, Uint8 *blue)
+int SDL_GetGammaRamp(Uint16 *red, Uint16 *green, Uint16 *blue)
 {
 	SDL_VideoDevice *video = current_video;
 	SDL_VideoDevice *this  = current_video;	
@@ -220,16 +216,14 @@ int SDL_GetGammaRamp(Uint8 *red, Uint8 *green, Uint8 *blue)
 		}
 		if ( video->GetGammaRamp ) {
 			/* Get the real hardware gamma */
-			video->GetGammaRamp(this, &video->gamma[0*256],
-			                          &video->gamma[1*256],
-			                          &video->gamma[2*256]);
+			video->GetGammaRamp(this, video->gamma);
 		} else {
 			/* Assume an identity gamma */
 			int i;
 			for ( i=0; i<256; ++i ) {
-				video->gamma[0*256+i] = i;
-				video->gamma[1*256+i] = i;
-				video->gamma[2*256+i] = i;
+				video->gamma[0*256+i] = (i << 8) | i;
+				video->gamma[1*256+i] = (i << 8) | i;
+				video->gamma[2*256+i] = (i << 8) | i;
 			}
 		}
 	}
