@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_stretch.c,v 1.1.2.7 2000/08/31 20:09:53 hercules Exp $";
+ "@(#) $Id: SDL_stretch.c,v 1.1.2.10 2001/02/22 00:21:51 hercules Exp $";
 #endif
 
 /* This a stretch blit implementation based on ideas given to me by
@@ -39,7 +39,7 @@ static char rcsid =
    into the general blitting mechanism.
 */
 
-#if (defined(WIN32) && !defined(_M_ALPHA)) || \
+#if (defined(WIN32) && !defined(_M_ALPHA) && !defined(_WIN32_WCE)) || \
     defined(i386) && defined(__GNUC__) && defined(USE_ASMBLIT)
 #define USE_ASM_STRETCH
 #endif
@@ -95,6 +95,7 @@ static int generate_rowbytes(int src_w, int dst_w, int bpp)
 		store = STORE_WORD;
 		break;
 	    default:
+		SDL_SetError("ASM stretch of %d bytes isn't supported\n", bpp);
 		return(-1);
 	}
 	pos = 0x10000;
@@ -144,13 +145,35 @@ void name(type *src, int src_w, type *dst, int dst_w)	\
 		pos += inc;				\
 	}						\
 }
-
 DEFINE_COPY_ROW(copy_row1, Uint8)
 DEFINE_COPY_ROW(copy_row2, Uint16)
 DEFINE_COPY_ROW(copy_row4, Uint32)
 
 #endif /* USE_ASM_STRETCH */
 
+/* The ASM code doesn't handle 24-bpp stretch blits */
+void copy_row3(Uint8 *src, int src_w, Uint8 *dst, int dst_w)
+{
+	int i;
+	int pos, inc;
+	Uint8 pixel[3];
+
+	pos = 0x10000;
+	inc = (src_w << 16) / dst_w;
+	for ( i=dst_w; i>0; --i ) {
+		while ( pos >= 0x10000L ) {
+			pixel[0] = *src++;
+			pixel[1] = *src++;
+			pixel[2] = *src++;
+			pos -= 0x10000L;
+		}
+		*dst++ = pixel[0];
+		*dst++ = pixel[1];
+		*dst++ = pixel[2];
+		pos += inc;
+	}
+}
+
 /* Perform a stretch blit between two surfaces of the same format.
    NOTE:  This function is not safe to call from multiple threads!
 */
@@ -158,7 +181,6 @@ int SDL_SoftStretch(SDL_Surface *src, SDL_Rect *srcrect,
                     SDL_Surface *dst, SDL_Rect *dstrect)
 {
 	int pos, inc;
-	int bpp;
 	int dst_width;
 	int dst_maxrow;
 	int src_row, dst_row;
@@ -169,11 +191,8 @@ int SDL_SoftStretch(SDL_Surface *src, SDL_Rect *srcrect,
 #if defined(USE_ASM_STRETCH) && defined(__GNUC__)
 	int u1, u2;
 #endif
+	const int bpp = dst->format->BytesPerPixel;
 
-	if ( dst->format->BytesPerPixel == 3 ) {
-		SDL_SetError("Only works with 8, 16, 32 bpp surfaces");
-		return(-1);
-	}
 	if ( src->format->BitsPerPixel != dst->format->BitsPerPixel ) {
 		SDL_SetError("Only works with same format surfaces");
 		return(-1);
@@ -209,19 +228,21 @@ int SDL_SoftStretch(SDL_Surface *src, SDL_Rect *srcrect,
 		dstrect = &full_dst;
 	}
 
-#ifdef USE_ASM_STRETCH
-	/* Write the opcodes for this stretch */
-	generate_rowbytes(srcrect->w, dstrect->w, src->format->BytesPerPixel);
-#endif
-
 	/* Set up the data... */
 	pos = 0x10000;
 	inc = (srcrect->h << 16) / dstrect->h;
 	src_row = srcrect->y;
 	dst_row = dstrect->y;
-	bpp = dst->format->BytesPerPixel;
 	dst_width = dstrect->w*bpp;
 
+#ifdef USE_ASM_STRETCH
+	/* Write the opcodes for this stretch */
+	if ( (bpp != 3) &&
+	     (generate_rowbytes(srcrect->w, dstrect->w, bpp) < 0) ) {
+		return(-1);
+	}
+#endif
+
 	/* Perform the stretch blit */
 	for ( dst_maxrow = dst_row+dstrect->h; dst_row<dst_maxrow; ++dst_row ) {
 		dstp = (Uint8 *)dst->pixels + (dst_row*dst->pitch)
@@ -233,32 +254,39 @@ int SDL_SoftStretch(SDL_Surface *src, SDL_Rect *srcrect,
 			pos -= 0x10000L;
 		}
 #ifdef USE_ASM_STRETCH
+		switch (bpp) {
+		    case 3:
+			copy_row3(srcp, srcrect->w, dstp, dstrect->w);
+			break;
+		    default:
 #ifdef __GNUC__
-		__asm__ __volatile__ ("
-			call _copy_row
-		"
-		: "=&D" (u1), "=&S" (u2)
-		: "0" (dstp), "1" (srcp)
-		: "memory" );
+			__asm__ __volatile__ ("
+				call _copy_row
+			"
+			: "=&D" (u1), "=&S" (u2)
+			: "0" (dstp), "1" (srcp)
+			: "memory" );
 #else
 #ifdef WIN32
-	{ void *code = &copy_row;
-		__asm {
-			push edi
-			push esi
+		{ void *code = &copy_row;
+			__asm {
+				push edi
+				push esi
+	
+				mov edi, dstp
+				mov esi, srcp
+				call dword ptr code
 
-			mov edi, dstp
-			mov esi, srcp
-			call dword ptr code
-
-			pop esi
-			pop edi
+				pop esi
+				pop edi
+			}
 		}
-	}
 #else
 #error Need inline assembly for this compiler
 #endif
 #endif /* __GNUC__ */
+			break;
+		}
 #else
 		switch (bpp) {
 		    case 1:
@@ -268,6 +296,9 @@ int SDL_SoftStretch(SDL_Surface *src, SDL_Rect *srcrect,
 			copy_row2((Uint16 *)srcp, srcrect->w,
 			          (Uint16 *)dstp, dstrect->w);
 			break;
+		    case 3:
+			copy_row3(srcp, srcrect->w, dstp, dstrect->w);
+			break;
 		    case 4:
 			copy_row4((Uint32 *)srcp, srcrect->w,
 			          (Uint32 *)dstp, dstrect->w);
