@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,18 +22,36 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_sysvideo.h,v 1.13 2000/02/12 23:51:41 hercules Exp $";
+ "@(#) $Id: SDL_sysvideo.h,v 1.6.2.45 2001/02/10 07:20:05 hercules Exp $";
 #endif
 
 #ifndef _SDL_sysvideo_h
 #define _SDL_sysvideo_h
 
+#include "SDL_mouse.h"
+#define SDL_PROTOTYPES_ONLY
+#include "SDL_syswm.h"
+#undef SDL_PROTOTYPES_ONLY
+
 /* This file prototypes the video driver implementation.
    This is designed to be easily converted to C++ in the future.
  */
 
-#include "SDL_mouse.h"
-#include "SDL_syswm.h"
+/* OpenGL is pretty much available on all Windows systems */
+#ifdef WIN32
+#ifndef _WIN32_WCE
+#define HAVE_OPENGL
+#endif
+#include <windows.h>
+#endif
+
+#ifdef HAVE_OPENGL
+#ifdef MACOSX
+#include <OpenGL/gl.h>  /* OpenGL.framework */
+#else
+#include <GL/gl.h>
+#endif /* MACOSX */
+#endif /* HAVE_OPENGL */
 
 /* The SDL video driver */
 typedef struct SDL_VideoDevice SDL_VideoDevice;
@@ -72,9 +90,7 @@ struct SDL_VideoDevice {
 	SDL_Surface *(*SetVideoMode)(_THIS, SDL_Surface *current,
 				int width, int height, int bpp, Uint32 flags);
 
-	/* Toggle the fullscreen mode.
-	   This function returns 1 if succeeds, or 0 if it fails.
-	*/
+	/* Toggle the fullscreen mode */
 	int (*ToggleFullScreen)(_THIS, int on);
 
 	/* This is called after the video mode has been set, to get the
@@ -83,13 +99,23 @@ struct SDL_VideoDevice {
 	 */
 	void (*UpdateMouse)(_THIS);
 
-	/* Sets the system palette with 
-	   SDL_Palette *palette = screen->format->palette;
-		'palette->colors[firstcolor] ... palette->colors[ncolors-1]'
-	   and returns 1 if all entries were set properly, or 0 otherwise, with
-	   the palette colors set to the actual colors that were set.
+	/* Create a YUV video surface (possibly overlay) of the given
+	   format.  The hardware should be able to perform at least 2x
+	   scaling on display.
+	 */
+	SDL_Overlay *(*CreateYUVOverlay)(_THIS, int width, int height,
+	                                 Uint32 format, SDL_Surface *display);
+
+        /* Sets the color entries { firstcolor .. (firstcolor+ncolors-1) }
+	   of the physical palette to those in 'colors'. If the device is
+	   using a software palette (SDL_HWPALETTE not set), then the
+	   changes are reflected in the logical palette of the screen
+	   as well.
+	   The return value is 1 if all entries could be set properly
+	   or 0 otherwise.
 	*/
-	int (*SetColors)(_THIS, int firstcolor, int ncolors);
+	int (*SetColors)(_THIS, int firstcolor, int ncolors,
+			 SDL_Color *colors);
 
 	/* This pointer should exist in the native video subsystem and should
 	   point to an appropriate update function for the current video mode
@@ -135,14 +161,63 @@ struct SDL_VideoDevice {
 	void (*FreeHWSurface)(_THIS, SDL_Surface *surface);
 
 	/* * * */
-	/* Window manager functions */
+	/* Gamma support */
 
-	/* Set the window icon image */
-	void (*SetIcon)(_THIS, SDL_Surface *icon, Uint8 *mask);
+	Uint16 *gamma;
+
+	/* Set the gamma correction directly (emulated with gamma ramps) */
+	int (*SetGamma)(_THIS, float red, float green, float blue);
+
+	/* Get the gamma correction directly (emulated with gamma ramps) */
+	int (*GetGamma)(_THIS, float *red, float *green, float *blue);
+
+	/* Set the gamma ramp */
+	int (*SetGammaRamp)(_THIS, Uint16 *ramp);
+
+	/* Get the gamma ramp */
+	int (*GetGammaRamp)(_THIS, Uint16 *ramp);
+
+	/* * * */
+	/* OpenGL support */
+
+	/* Sets the dll to use for OpenGL and loads it */
+	int (*GL_LoadLibrary)(_THIS, const char *path);
+
+	/* Retrieves the address of a function in the gl library */
+	void* (*GL_GetProcAddress)(_THIS, const char *proc);
+
+        /* Get attribute information from the windowing system. */
+        int (*GL_GetAttribute)(_THIS, SDL_GLattr attrib, int* value);
+
+        /* Make the context associated with this driver current */
+        int (*GL_MakeCurrent)(_THIS);
+
+	/* Swap the current buffers in double buffer mode. */
+	void (*GL_SwapBuffers)(_THIS);
+
+  	/* OpenGL functions for SDL_OPENGLBLIT */
+#ifdef HAVE_OPENGL
+#ifndef WIN32
+#define WINAPI
+#endif
+#define SDL_PROC(ret,func,params) ret (WINAPI *func) params;
+#include "SDL_glfuncs.h"
+#undef SDL_PROC
+
+	/* Texture id */
+	GLuint texture;
+#endif
+	int is_32bit;
+ 
+	/* * * */
+	/* Window manager functions */
 
 	/* Set the title and icon text */
 	void (*SetCaption)(_THIS, const char *title, const char *icon);
 
+	/* Set the window icon image */
+	void (*SetIcon)(_THIS, SDL_Surface *icon, Uint8 *mask);
+
 	/* Iconify the window.
 	   This function returns 1 if there is a window manager and the
 	   window was actually iconified, it returns 0 otherwise.
@@ -158,19 +233,26 @@ struct SDL_VideoDevice {
 	/* * * */
 	/* Cursor manager functions */
 
-	/* Free a window manager cursor */
+	/* Free a window manager cursor
+	   This function can be NULL if CreateWMCursor is also NULL.
+	 */
 	void (*FreeWMCursor)(_THIS, WMcursor *cursor);
 
-	/* Create a black/white window manager cursor */
+	/* If not NULL, create a black/white window manager cursor */
 	WMcursor *(*CreateWMCursor)(_THIS,
 		Uint8 *data, Uint8 *mask, int w, int h, int hot_x, int hot_y);
 
-	/* Show or the specified cursor, or hide if NULL */
+	/* Show the specified cursor, or hide if cursor is NULL */
 	int (*ShowWMCursor)(_THIS, WMcursor *cursor);
 
-	/* Warp the window manager cursor to (x,y) */
+	/* Warp the window manager cursor to (x,y)
+	   If NULL, a mouse motion event is posted internally.
+	 */
 	void (*WarpWMCursor)(_THIS, Uint16 x, Uint16 y);
 
+	/* If not NULL, this is called when a mouse motion event occurs */
+	void (*MoveWMCursor)(_THIS, int x, int y);
+
 	/* Determine whether the mouse should be in relative mode or not.
 	   This function is called when the input grab state or cursor
 	   visibility state changes.
@@ -194,6 +276,8 @@ struct SDL_VideoDevice {
 	SDL_Surface *screen;
 	SDL_Surface *shadow;
 	SDL_Surface *visible;
+        SDL_Palette *physpal;	/* physical palette, if != logical palette */
+        SDL_Color *gammacols;	/* gamma-corrected colours, or NULL */
 	char *wm_title;
 	char *wm_icon;
 	int offset_x;
@@ -203,9 +287,30 @@ struct SDL_VideoDevice {
 	/* Driver information flags */
 	int handles_any_size;	/* Driver handles any size video mode */
 
+	/* * * */
+	/* Data used by the GL drivers */
+	struct {
+		int red_size;
+		int green_size;
+		int blue_size;
+		int alpha_size;
+		int depth_size;
+	        int buffer_size;
+		int stencil_size;
+		int double_buffer;
+                int accum_red_size;
+                int accum_green_size;
+                int accum_blue_size;
+                int accum_alpha_size;
+		int driver_loaded;
+		char driver_path[256];
+		void* dll_handle;
+	} gl_config;
+
 	/* * * */
 	/* Data private to this driver */
 	struct SDL_PrivateVideoData *hidden;
+	struct SDL_PrivateGLData *gl_data;
 
 	/* * * */
 	/* The function used to dispose of this structure */
@@ -215,6 +320,7 @@ struct SDL_VideoDevice {
 
 typedef struct VideoBootStrap {
 	const char *name;
+	const char *desc;
 	int (*available)(void);
 	SDL_VideoDevice *(*create)(int devindex);
 } VideoBootStrap;
@@ -222,15 +328,24 @@ typedef struct VideoBootStrap {
 #ifdef ENABLE_X11
 extern VideoBootStrap X11_bootstrap;
 #endif
+#ifdef ENABLE_DGA
+extern VideoBootStrap DGA_bootstrap;
+#endif
 #ifdef ENABLE_FBCON
 extern VideoBootStrap FBCON_bootstrap;
 #endif
+#ifdef ENABLE_PS2GS
+extern VideoBootStrap PS2GS_bootstrap;
+#endif
 #ifdef ENABLE_GGI
 extern VideoBootStrap GGI_bootstrap;
 #endif
 #ifdef ENABLE_SVGALIB
 extern VideoBootStrap SVGALIB_bootstrap;
 #endif
+#ifdef ENABLE_AALIB
+extern VideoBootStrap AALIB_bootstrap;
+#endif
 #ifdef ENABLE_WINDIB
 extern VideoBootStrap WINDIB_bootstrap;
 #endif
@@ -240,12 +355,22 @@ extern VideoBootStrap DIRECTX_bootstrap;
 #ifdef ENABLE_BWINDOW
 extern VideoBootStrap BWINDOW_bootstrap;
 #endif
-#ifdef macintosh
+/* MacOS X gets the proper defines from configure */
+#if defined(macintosh) && !defined(MACOSX)
 #define ENABLE_TOOLBOX
+#if !TARGET_API_MAC_CARBON
+#define ENABLE_DRAWSPROCKET
+#endif
 #endif
 #ifdef ENABLE_TOOLBOX
 extern VideoBootStrap TOOLBOX_bootstrap;
 #endif
+#ifdef ENABLE_DRAWSPROCKET
+extern VideoBootStrap DSp_bootstrap;
+#endif
+#ifdef ENABLE_CYBERGRAPHICS
+extern VideoBootStrap CGX_bootstrap;
+#endif
 
 /* This is the current video device */
 extern SDL_VideoDevice *current_video;
