@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,15 +22,9 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_ggimouse_c.h,v 1.1.2.2 2000/08/24 19:35:34 hercules Exp $";
+ "@(#) $Id: SDL_ggimouse_c.h,v 1.1.2.4 2001/02/10 07:20:06 hercules Exp $";
 #endif
 
 #include "SDL_ggivideo.h"
 
 /* Functions to be exported */
-extern void GGI_FreeWMCursor(_THIS, WMcursor *cursor);
-extern WMcursor *GGI_CreateWMCursor(_THIS,
-		Uint8 *data, Uint8 *mask, int w, int h, int hot_x, int hot_y);
-extern int GGI_ShowWMCursor(_THIS, WMcursor *cursor);
-extern void GGI_WarpWMCursor(_THIS, Uint16 x, Uint16 y);
-
