@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_macmouse_c.h,v 1.1.2.1 2000/05/30 15:59:58 hercules Exp $";
+ "@(#) $Id: SDL_macmouse_c.h,v 1.1.2.3 2001/02/10 07:20:06 hercules Exp $";
 #endif
 
 #include "SDL_romvideo.h"
@@ -34,3 +34,5 @@ extern WMcursor *Mac_CreateWMCursor(_THIS,
 extern int Mac_ShowWMCursor(_THIS, WMcursor *cursor);
 extern void Mac_WarpWMCursor(_THIS, Uint16 x, Uint16 y);
 
+/* Data to be exported */
+extern int Mac_cursor_showing;
