@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_svgamouse.c,v 1.1.2.1 2000/03/16 15:20:39 hercules Exp $";
+ "@(#) $Id: SDL_svgamouse.c,v 1.1.2.3 2001/02/10 07:20:07 hercules Exp $";
 #endif
 
 #include <stdio.h>
@@ -38,29 +38,3 @@ static char rcsid =
 struct WMcursor {
 	int unused;
 };
-
-
-void SVGA_FreeWMCursor(_THIS, WMcursor *cursor)
-{
-	return;
-}
-
-WMcursor *SVGA_CreateWMCursor(_THIS,
-		Uint8 *data, Uint8 *mask, int w, int h, int hot_x, int hot_y)
-{
-	return(NULL);
-}
-
-int SVGA_ShowWMCursor(_THIS, WMcursor *cursor)
-{
-	return(0);
-}
-
-void SVGA_WarpWMCursor(_THIS, Uint16 x, Uint16 y)
-{
-	x += (this->screen->offset % this->screen->pitch) /
-	      this->screen->format->BytesPerPixel;
-	y += (this->screen->offset / this->screen->pitch);
-	SDL_PrivateMouseMotion(0, 0, x, y);
-}
-
