@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,15 +22,9 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_svgamouse_c.h,v 1.2 1999/12/17 19:10:43 hercules Exp $";
+ "@(#) $Id: SDL_svgamouse_c.h,v 1.2.2.3 2001/02/10 07:20:07 hercules Exp $";
 #endif
 
 #include "SDL_svgavideo.h"
 
 /* Functions to be exported */
-extern void SVGA_FreeWMCursor(_THIS, WMcursor *cursor);
-extern WMcursor *SVGA_CreateWMCursor(_THIS,
-		Uint8 *data, Uint8 *mask, int w, int h, int hot_x, int hot_y);
-extern int SVGA_ShowWMCursor(_THIS, WMcursor *cursor);
-extern void SVGA_WarpWMCursor(_THIS, Uint16 x, Uint16 y);
-
