@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,12 +22,12 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_dx5events.c,v 1.6 2000/01/17 05:26:23 hercules Exp $";
+ "@(#) $Id: SDL_dx5events.c,v 1.3.2.12 2001/02/10 07:20:09 hercules Exp $";
 #endif
 
 /* CAUTION!!!!  If you modify this file, check ../windib/SDL_sysevents.c */
 
-#include <directx.h>
+#include "directx.h"
 
 #include <stdio.h>
 #include "SDL_events.h"
@@ -57,7 +57,7 @@ static int mouse_pressed;
 
 /* The translation table from a DirectInput scancode to an SDL keysym */
 static SDLKey DIK_keymap[256];
-static SDL_keysym *TranslateKey(UINT scancode, SDL_keysym *keysym);
+static SDL_keysym *TranslateKey(UINT scancode, SDL_keysym *keysym, int pressed);
 
 /* Convert a DirectInput return code to a text message */
 static void SetDIerror(char *function, int code)
@@ -279,10 +279,10 @@ static void handle_keyboard(const int numevents, DIDEVICEOBJECTDATA *keybuf)
 	for ( i=0; i<numevents; ++i ) {
 		if ( keybuf[i].dwData & 0x80 ) {
 			posted = SDL_PrivateKeyboard(SDL_PRESSED,
-				    TranslateKey(keybuf[i].dwOfs, &keysym));
+				    TranslateKey(keybuf[i].dwOfs, &keysym, 1));
 		} else {
 			posted = SDL_PrivateKeyboard(SDL_RELEASED,
-				    TranslateKey(keybuf[i].dwOfs, &keysym));
+				    TranslateKey(keybuf[i].dwOfs, &keysym, 0));
 		}
 	}
 }
@@ -340,6 +340,9 @@ static void handle_mouse(const int numevents, DIDEVICEOBJECTDATA *ptrbuf)
 				 */
 				if ( button == 2 ) {
 					button = 3;
+				} else
+				if ( button == 3 ) {
+					button = 2;
 				}
 				if ( new_state & 0x01 ) {
 					/* Grab mouse so we get mouse-up */
@@ -392,6 +395,9 @@ static void handle_mouse(const int numevents, DIDEVICEOBJECTDATA *ptrbuf)
 				 */
 				if ( button == 2 ) {
 					button = 3;
+				} else
+				if ( button == 3 ) {
+					button = 2;
 				}
 				if ( ptrbuf[i].dwData & 0x80 ) {
 					/* Grab mouse so we get mouse-up */
@@ -438,14 +444,6 @@ LONG
 				}
 				mouse_lost = 1;
 			}
-			if ( SDL_PublicSurface &&
-			     (SDL_PublicSurface->flags & SDL_FULLSCREEN) ) {
-				posted = SDL_PrivateAppActive((Uint8)active,
-					(SDL_APPMOUSEFOCUS|SDL_APPINPUTFOCUS));
-			} else {
-				posted = SDL_PrivateAppActive((Uint8)active,
-							SDL_APPINPUTFOCUS);
-			}
 		}
 		break;
 
@@ -714,17 +712,16 @@ void DX5_InitOSKeymap(_THIS)
 	DIK_keymap[DIK_LWIN] = SDLK_LMETA;
 	DIK_keymap[DIK_RWIN] = SDLK_RMETA;
 	DIK_keymap[DIK_APPS] = SDLK_MENU;
-
 }
 
-static SDL_keysym *TranslateKey(UINT scancode, SDL_keysym *keysym)
+static SDL_keysym *TranslateKey(UINT scancode, SDL_keysym *keysym, int pressed)
 {
 	/* Set the keysym information */
 	keysym->scancode = (unsigned char)scancode;
 	keysym->sym = DIK_keymap[scancode];
 	keysym->mod = KMOD_NONE;
 	keysym->unicode = 0;
-	if ( SDL_TranslateUNICODE ) { /* Someday use ToUnicode() */
+	if ( pressed && SDL_TranslateUNICODE ) { /* Someday use ToUnicode() */
 		UINT vkey;
 		BYTE keystate[256];
 		BYTE chars[2];
@@ -750,14 +747,19 @@ int DX5_CreateWindow(_THIS)
 	}
 
 	/* Create the SDL window */
-	SDL_Window = CreateWindow(SDL_Appname, SDL_Appname,
+	SDL_RegisterApp("SDL_app", CS_BYTEALIGNCLIENT, 0);
+	if ( SDL_windowid ) {
+		SDL_Window = (HWND)strtol(SDL_windowid, NULL, 0);
+	} else {
+		SDL_Window = CreateWindow(SDL_Appname, SDL_Appname,
                         (WS_OVERLAPPED|WS_CAPTION|WS_SYSMENU|WS_MINIMIZEBOX),
                                  0, 0, 0, 0, NULL, NULL, SDL_Instance, NULL);
-	if ( SDL_Window == NULL ) {
-		SDL_SetError("Couldn't create window");
-		return(-1);
+		if ( SDL_Window == NULL ) {
+			SDL_SetError("Couldn't create window");
+			return(-1);
+		}
+		ShowWindow(SDL_Window, SW_HIDE);
 	}
-	ShowWindow(SDL_Window, SW_HIDE);
 
 	/* Initialize DirectInput */
 	if ( DX5_DInputInit(this) < 0 ) {
