@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,14 +22,19 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_dx5video.c,v 1.14 2000/02/23 11:03:54 hercules Exp $";
+ "@(#) $Id: SDL_dx5video.c,v 1.11.2.37 2001/02/28 12:35:28 hercules Exp $";
 #endif
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <malloc.h>
 #include <windows.h>
-#include <directx.h>
+#include "directx.h"
+
+/* Not yet in the mingw32 cross-compile headers */
+#ifndef CDS_FULLSCREEN
+#define CDS_FULLSCREEN	4
+#endif
 
 #include "SDL_error.h"
 #include "SDL_timer.h"
@@ -39,16 +44,28 @@ static char rcsid =
 #include "SDL_RLEaccel_c.h"
 #include "SDL_blit.h"
 #include "SDL_pixels_c.h"
+#include "SDL_dx5video.h"
 #include "SDL_syswm_c.h"
 #include "SDL_sysmouse_c.h"
 #include "SDL_dx5events_c.h"
-#include "SDL_dx5video.h"
+#include "SDL_dx5yuv_c.h"
+#include "SDL_wingl_c.h"
 
+/* Function called to create a GL video mode - use the GDI driver, not DirectX */
+extern SDL_Surface *DIB_SetVideoMode(_THIS, SDL_Surface *current,
+				int width, int height, int bpp, Uint32 flags);
 
 /* DirectX function pointers for video and events */
 HRESULT (WINAPI *DDrawCreate)( GUID FAR *lpGUID, LPDIRECTDRAW FAR *lplpDD, IUnknown FAR *pUnkOuter );
 HRESULT (WINAPI *DInputCreate)(HINSTANCE hinst, DWORD dwVersion, LPDIRECTINPUTA *ppDI, LPUNKNOWN punkOuter);
 
+/* This is the rect EnumModes2 uses */
+struct DX5EnumRect {
+	SDL_Rect r;
+	struct DX5EnumRect* next;
+};
+static struct DX5EnumRect *enumlists[NUM_MODELISTS];
+
 /*
  * Experimentally determined values for c_cfDI* constants used in DirectX 5.0
  */
@@ -388,7 +405,13 @@ const DIDATAFORMAT c_dfDIJoystick = { 24, 16, 0x00000001, 80, 44, JOY_fmt };
 static int DX5_VideoInit(_THIS, SDL_PixelFormat *vformat);
 static SDL_Rect **DX5_ListModes(_THIS, SDL_PixelFormat *format, Uint32 flags);
 static SDL_Surface *DX5_SetVideoMode(_THIS, SDL_Surface *current, int width, int height, int bpp, Uint32 flags);
-static int DX5_SetColors(_THIS, int firstcolor, int ncolors);
+static int DX5_SetColors(_THIS, int firstcolor, int ncolors,
+			 SDL_Color *colors);
+static void DX5_SwapGamma(_THIS);
+#ifdef IID_IDirectDrawGammaControl
+static int DX5_SetGammaRamp(_THIS, Uint16 *ramp);
+static int DX5_GetGammaRamp(_THIS, Uint16 *ramp);
+#endif
 static void DX5_VideoQuit(_THIS);
 
 /* Hardware surface functions */
@@ -515,8 +538,15 @@ static void DX5_DeleteDevice(SDL_VideoDevice *this)
 		IDirectDraw2_Release(ddraw2);
 	}
 	DX5_Unload();
-	free(this->hidden);
-	free(this);
+	if ( this ) {
+		if ( this->hidden ) {
+			free(this->hidden);
+		}
+		if ( this->gl_data ) {
+			free(this->gl_data);
+		}
+		free(this);
+	}
 }
 
 static SDL_VideoDevice *DX5_CreateDevice(int devindex)
@@ -534,21 +564,24 @@ static SDL_VideoDevice *DX5_CreateDevice(int devindex)
 		memset(device, 0, (sizeof *device));
 		device->hidden = (struct SDL_PrivateVideoData *)
 				malloc((sizeof *device->hidden));
+		device->gl_data = (struct SDL_PrivateGLData *)
+				malloc((sizeof *device->gl_data));
 	}
-	if ( (device == NULL) || (device->hidden == NULL) ) {
+	if ( (device == NULL) || (device->hidden == NULL) ||
+		                 (device->gl_data == NULL) ) {
 		SDL_OutOfMemory();
-		if ( device ) {
-			free(device);
-		}
-		return(0);
+		DX5_DeleteDevice(device);
+		return(NULL);
 	}
 	memset(device->hidden, 0, (sizeof *device->hidden));
+	memset(device->gl_data, 0, (sizeof *device->gl_data));
 
 	/* Set the function pointers */
 	device->VideoInit = DX5_VideoInit;
 	device->ListModes = DX5_ListModes;
 	device->SetVideoMode = DX5_SetVideoMode;
 	device->UpdateMouse = WIN_UpdateMouse;
+	device->CreateYUVOverlay = DX5_CreateYUVOverlay;
 	device->SetColors = DX5_SetColors;
 	device->UpdateRects = NULL;
 	device->VideoQuit = DX5_VideoQuit;
@@ -561,8 +594,19 @@ static SDL_VideoDevice *DX5_CreateDevice(int devindex)
 	device->UnlockHWSurface = DX5_UnlockHWSurface;
 	device->FlipHWSurface = DX5_FlipHWSurface;
 	device->FreeHWSurface = DX5_FreeHWSurface;
-	device->SetIcon = WIN_SetWMIcon;
+#ifdef IID_IDirectDrawGammaControl
+	device->SetGammaRamp = DX5_SetGammaRamp;
+	device->GetGammaRamp = DX5_GetGammaRamp;
+#endif
+#ifdef HAVE_OPENGL
+        device->GL_LoadLibrary = WIN_GL_LoadLibrary;
+        device->GL_GetProcAddress = WIN_GL_GetProcAddress;
+        device->GL_GetAttribute = WIN_GL_GetAttribute;
+        device->GL_MakeCurrent = WIN_GL_MakeCurrent;
+        device->GL_SwapBuffers = WIN_GL_SwapBuffers;
+#endif
 	device->SetCaption = WIN_SetWMCaption;
+	device->SetIcon = WIN_SetWMIcon;
 	device->IconifyWindow = WIN_IconifyWindow;
 	device->GrabInput = WIN_GrabInput;
 	device->GetWMInfo = WIN_GetWMInfo;
@@ -577,6 +621,7 @@ static SDL_VideoDevice *DX5_CreateDevice(int devindex)
 	/* Set up the windows message handling functions */
 	WIN_RealizePalette = DX5_RealizePalette;
 	WIN_PaletteChanged = DX5_PaletteChanged;
+	WIN_SwapGamma = DX5_SwapGamma;
 	WIN_WinPAINT = DX5_WinPAINT;
 	HandleMessage = DX5_HandleMessage;
 
@@ -587,37 +632,19 @@ static SDL_VideoDevice *DX5_CreateDevice(int devindex)
 }
 
 VideoBootStrap DIRECTX_bootstrap = {
-	"directx", DX5_Available, DX5_CreateDevice
+	"directx", "Win95/98/2000 DirectX",
+	DX5_Available, DX5_CreateDevice
 };
 
-static HRESULT WINAPI CountModes(DDSURFACEDESC *desc, VOID *udata)
+static HRESULT WINAPI EnumModes2(DDSURFACEDESC *desc, VOID *udata)
 {
 	SDL_VideoDevice *this = (SDL_VideoDevice *)udata;
+	struct DX5EnumRect *enumrect;
 #if defined(NONAMELESSUNION)
 	int bpp = desc->ddpfPixelFormat.u1.dwRGBBitCount;
 #else
 	int bpp = desc->ddpfPixelFormat.dwRGBBitCount;
 #endif
-	switch (bpp)  {
-		case 8:
-		case 16:
-		case 24:
-		case 32:
-			++SDL_nummodes[(bpp/8)-1];
-			break;
-	}
-	return(DDENUMRET_OK);
-}
-
-static HRESULT WINAPI EnumModes(DDSURFACEDESC *desc, VOID *udata)
-{
-	SDL_VideoDevice *this = (SDL_VideoDevice *)udata;
-#if defined(NONAMELESSUNION)
-	int bpp = desc->ddpfPixelFormat.u1.dwRGBBitCount;
-#else
-	int bpp = desc->ddpfPixelFormat.dwRGBBitCount;
-#endif
-	int index;
 
 	switch (bpp)  {
 		case 8:
@@ -625,22 +652,26 @@ static HRESULT WINAPI EnumModes(DDSURFACEDESC *desc, VOID *udata)
 		case 24:
 		case 32:
 			bpp /= 8; --bpp;
-			index = SDL_nummodes[bpp]-(++SDL_modeindex[bpp]);
-			SDL_modelist[bpp][index]->x = 0;
-			SDL_modelist[bpp][index]->y = 0;
-			SDL_modelist[bpp][index]->w = (Uint16)desc->dwWidth;
-			SDL_modelist[bpp][index]->h = (Uint16)desc->dwHeight;
-			break;
-		default:
-#ifdef DDRAW_DEBUG
-			fprintf(stderr, "Unknown bit-depth: %d\n", bpp);
-#endif
+			++SDL_nummodes[bpp];
+			enumrect = (struct DX5EnumRect*)malloc(sizeof(struct DX5EnumRect));
+			if ( !enumrect ) {
+				SDL_OutOfMemory();
+				return(DDENUMRET_CANCEL);
+			}
+			enumrect->r.x = 0;
+			enumrect->r.y = 0;
+			enumrect->r.w = (Uint16)desc->dwWidth;
+			enumrect->r.h = (Uint16)desc->dwHeight;
+			enumrect->next = enumlists[bpp];
+			enumlists[bpp] = enumrect;
 			break;
 	}
+
+
 	return(DDENUMRET_OK);
 }
 
-static void SetDDerror(char *function, int code)
+void SetDDerror(const char *function, int code)
 {
 	static char *error;
 	static char  errbuf[BUFSIZ];
@@ -746,6 +777,9 @@ static void SetDDerror(char *function, int code)
 		case DDERR_UNSUPPORTED:
 			error = "Operation not supported";
 			break;
+		case E_NOINTERFACE:
+			error = "Interface not present";
+			break;
 		default:
 			sprintf(errbuf, "%s: Unknown DirectDraw error: 0x%x",
 								function, code);
@@ -758,12 +792,15 @@ static void SetDDerror(char *function, int code)
 	return;
 }
 
+
 static int DX5_UpdateVideoInfo(_THIS)
 {
 	/* This needs to be DDCAPS_DX5 for the DirectDraw2 interface */
 #if DIRECTDRAW_VERSION <= 0x300
 #error Your version of DirectX must be greater than or equal to 5.0
-#elif DIRECTDRAW_VERSION <= 0x500
+#endif
+#ifndef IID_IDirectDrawGammaControl
+	/*if gamma is undefined then we really have directx <= 0x500*/
 	DDCAPS DDCaps;
 #else
 	DDCAPS_DX5 DDCaps;
@@ -862,33 +899,28 @@ int DX5_VideoInit(_THIS, SDL_PixelFormat *vformat)
 	ReleaseDC(SDL_Window, hdc);
 
 	/* Enumerate the available fullscreen modes */
-	result = IDirectDraw2_EnumDisplayModes(ddraw2,0,NULL,this,CountModes);
+	for ( i=0; i<NUM_MODELISTS; ++i )
+		enumlists[i] = NULL;
+
+	result = IDirectDraw2_EnumDisplayModes(ddraw2,0,NULL,this,EnumModes2);
 	if ( result != DD_OK ) {
 		SetDDerror("DirectDraw2::EnumDisplayModes", result);
 		return(-1);
 	}
 	for ( i=0; i<NUM_MODELISTS; ++i ) {
+		struct DX5EnumRect *rect;
 		SDL_modelist[i] = (SDL_Rect **)
 				malloc((SDL_nummodes[i]+1)*sizeof(SDL_Rect *));
 		if ( SDL_modelist[i] == NULL ) {
 			SDL_OutOfMemory();
 			return(-1);
 		}
-		for ( j=0; j<SDL_nummodes[i]; ++j ) {
-			SDL_modelist[i][j]=(SDL_Rect *)malloc(sizeof(SDL_Rect));
-			if ( SDL_modelist[i][j] == NULL ) {
-				SDL_OutOfMemory();
-				return(-1);
-			}
+		for ( j = 0, rect = enumlists[i]; rect; ++j, rect = rect->next ) {
+			SDL_modelist[i][j]=(SDL_Rect *)rect;
 		}
 		SDL_modelist[i][j] = NULL;
 	}
-	result = IDirectDraw2_EnumDisplayModes(ddraw2,0,NULL,this,EnumModes);
-	if ( result != DD_OK ) {
-		SetDDerror("DirectDraw2::EnumDisplayModes", result);
-		return(-1);
-	}
-
+	
 	/* Fill in some window manager capabilities */
 	this->info.wm_available = 1;
 
@@ -937,17 +969,23 @@ SDL_Surface *DX5_SetVideoMode(_THIS, SDL_Surface *current,
 	HRESULT result;
 	DWORD sharemode;
 	DWORD style;
-	const DWORD windowstyle = 
-			(WS_OVERLAPPED|WS_CAPTION|WS_SYSMENU|WS_MINIMIZEBOX);
 	const DWORD directstyle =
 			(WS_POPUP);
+	const DWORD windowstyle = 
+			(WS_OVERLAPPED|WS_CAPTION|WS_SYSMENU|WS_MINIMIZEBOX);
+	const DWORD resizestyle =
+			(WS_THICKFRAME|WS_MAXIMIZEBOX);
 	DDSURFACEDESC ddsd;
 	LPDIRECTDRAWSURFACE  dd_surface1;
 	LPDIRECTDRAWSURFACE3 dd_surface3;
+	BOOL was_visible;
 
 #ifdef DDRAW_DEBUG
  fprintf(stderr, "Setting %dx%dx%d video mode\n", width, height, bpp);
 #endif
+	/* See whether or not we should center the window */
+	was_visible = IsWindowVisible(SDL_Window);
+
 	/* Clean up any previous DirectDraw surfaces */
 	if ( current->hwdata ) {
 		this->FreeHWSurface(this, current);
@@ -958,14 +996,163 @@ SDL_Surface *DX5_SetVideoMode(_THIS, SDL_Surface *current,
 		SDL_primary = NULL;
 	}
 
+	/* Unset any previous OpenGL fullscreen mode */
+	if ( (current->flags & (SDL_OPENGL|SDL_FULLSCREEN)) ==
+	                       (SDL_OPENGL|SDL_FULLSCREEN) ) {
+		ChangeDisplaySettings(NULL, 0);
+	}
+
+	/* Clean up any GL context that may be hanging around */
+	if ( current->flags & SDL_OPENGL ) {
+		WIN_GL_ShutDown(this);
+	}
+
+	/* If we are setting a GL mode, use GDI, not DirectX (yuck) */
+	if ( flags & SDL_OPENGL ) {
+		RECT bounds;
+		int x, y;
+		Uint32 Rmask, Gmask, Bmask;
+
+		/* Recalculate the bitmasks if necessary */
+		if ( bpp == current->format->BitsPerPixel ) {
+			video = current;
+		} else {
+			switch (bpp) {
+			    case 15:
+			    case 16:
+				if ( 0 /*DIB_SussScreenDepth() == 15*/ ) {
+					/* 5-5-5 */
+					Rmask = 0x00007c00;
+					Gmask = 0x000003e0;
+					Bmask = 0x0000001f;
+				} else {
+					/* 5-6-5 */
+					Rmask = 0x0000f800;
+					Gmask = 0x000007e0;
+					Bmask = 0x0000001f;
+				}
+				break;
+			    case 24:
+			    case 32:
+				/* GDI defined as 8-8-8 */
+				Rmask = 0x00ff0000;
+				Gmask = 0x0000ff00;
+				Bmask = 0x000000ff;
+				break;
+			    default:
+				Rmask = 0x00000000;
+				Gmask = 0x00000000;
+				Bmask = 0x00000000;
+				break;
+			}
+			video = SDL_CreateRGBSurface(SDL_SWSURFACE, 0, 0, bpp,
+			                             Rmask, Gmask, Bmask, 0);
+			if ( video == NULL ) {
+				SDL_OutOfMemory();
+				return(NULL);
+			}
+		}
+
+		/* Fill in part of the video surface */
+		video->flags = 0;	/* Clear flags */
+		video->w = width;
+		video->h = height;
+		video->pitch = SDL_CalculatePitch(video);
+
+		/* Set fullscreen mode if appropriate.
+		   Ugh, since our list of valid video modes comes from
+		   the DirectX driver, we may not actually be able to
+		   change to the desired resolution here.
+		   FIXME: Should we do a closest match?
+		 */
+		if ( (flags & SDL_FULLSCREEN) == SDL_FULLSCREEN ) {
+			DEVMODE settings;
+
+			memset(&settings, 0, sizeof(DEVMODE));
+			settings.dmSize = sizeof(DEVMODE);
+			settings.dmBitsPerPel = video->format->BitsPerPixel;
+			settings.dmPelsWidth = width;
+			settings.dmPelsHeight = height;
+			settings.dmFields = DM_PELSWIDTH | DM_PELSHEIGHT | DM_BITSPERPEL;
+			if ( ChangeDisplaySettings(&settings, CDS_FULLSCREEN) == DISP_CHANGE_SUCCESSFUL ) {
+				video->flags |= SDL_FULLSCREEN;
+			}
+		}
+
+		style = GetWindowLong(SDL_Window, GWL_STYLE);
+		style &= ~(resizestyle|WS_MAXIMIZE);
+		if ( (video->flags & SDL_FULLSCREEN) == SDL_FULLSCREEN ) {
+			style &= ~windowstyle;
+			style |= directstyle;
+		} else {
+			if ( flags & SDL_NOFRAME ) {
+				style &= ~windowstyle;
+				style |= directstyle;
+				video->flags |= SDL_NOFRAME;
+			} else {
+				style &= ~directstyle;
+				style |= windowstyle;
+				if ( flags & SDL_RESIZABLE ) {
+					style |= resizestyle;
+					video->flags |= SDL_RESIZABLE;
+				}
+			}
+			if (IsZoomed(SDL_Window)) style |= WS_MAXIMIZE;
+		}
+		SetWindowLong(SDL_Window, GWL_STYLE, style);
+
+		/* Resize the window (copied from SDL WinDIB driver) */
+		if ( SDL_windowid == NULL ) {
+			UINT swp_flags;
+
+			SDL_resizing = 1;
+			bounds.left = 0;
+			bounds.top = 0;
+			bounds.right = video->w;
+			bounds.bottom = video->h;
+			AdjustWindowRect(&bounds, GetWindowLong(SDL_Window, GWL_STYLE), FALSE);
+			width = bounds.right-bounds.left;
+			height = bounds.bottom-bounds.top;
+			x = (GetSystemMetrics(SM_CXSCREEN)-width)/2;
+			y = (GetSystemMetrics(SM_CYSCREEN)-height)/2;
+			if ( y < 0 ) { /* Cover up title bar for more client area */
+				y -= GetSystemMetrics(SM_CYCAPTION)/2;
+			}
+			swp_flags = (SWP_NOCOPYBITS | SWP_NOZORDER | SWP_SHOWWINDOW);
+			if ( was_visible && !(flags & SDL_FULLSCREEN) ) {
+				swp_flags |= SWP_NOMOVE;
+			}
+			SetWindowPos(SDL_Window, NULL, x, y, width, height, swp_flags);
+			SDL_resizing = 0;
+			SetForegroundWindow(SDL_Window);
+		}
+
+		/* Set up for OpenGL */
+		if ( WIN_GL_SetupWindow(this) < 0 ) {
+			return(NULL);
+		}
+		video->flags |= SDL_OPENGL;
+		return(video);
+	}
+
 	/* Set the appropriate window style */
 	style = GetWindowLong(SDL_Window, GWL_STYLE);
+	style &= ~(resizestyle|WS_MAXIMIZE);
 	if ( (flags & SDL_FULLSCREEN) == SDL_FULLSCREEN ) {
 		style &= ~windowstyle;
 		style |= directstyle;
 	} else {
-		style &= ~directstyle;
-		style |= windowstyle;
+		if ( flags & SDL_NOFRAME ) {
+			style &= ~windowstyle;
+			style |= directstyle;
+		} else {
+			style &= ~directstyle;
+			style |= windowstyle;
+			if ( flags & SDL_RESIZABLE ) {
+				style |= resizestyle;
+			}
+		}
+		if (IsZoomed(SDL_Window)) style |= WS_MAXIMIZE;
 	}
 	SetWindowLong(SDL_Window, GWL_STYLE, style);
 
@@ -1126,7 +1313,8 @@ SDL_Surface *DX5_SetVideoMode(_THIS, SDL_Surface *current,
 				ddsd.ddpfPixelFormat.u1.dwRGBBitCount) ) {
 #else
 				ddsd.ddpfPixelFormat.dwRGBBitCount) ) {
-#endif 
+#endif
+		SDL_FreeSurface(video);
 		video = SDL_CreateRGBSurface(SDL_SWSURFACE, 0, 0,
 #if defined(NONAMELESSUNION)
 				ddsd.ddpfPixelFormat.u1.dwRGBBitCount,
@@ -1146,9 +1334,7 @@ SDL_Surface *DX5_SetVideoMode(_THIS, SDL_Surface *current,
 		}
 		video->w = width;
 		video->h = height;
-		video->pitch = (width*video->format->BytesPerPixel);
-		/* Pitch needs to be QWORD (8-byte) aligned */
-		video->pitch = (video->pitch + 7) & ~7;
+		video->pitch = 0;
 	}
 	video->flags = 0;	/* Clear flags */
 
@@ -1170,7 +1356,10 @@ SDL_Surface *DX5_SetVideoMode(_THIS, SDL_Surface *current,
 	if ( (flags & SDL_FULLSCREEN) != SDL_FULLSCREEN ) {
 		/* Necessary if we're going from fullscreen to window */
 		if ( video->pixels == NULL ) {
-			video->pixels = (void *)malloc(video->w*video->pitch);
+			video->pitch = (width*video->format->BytesPerPixel);
+			/* Pitch needs to be QWORD (8-byte) aligned */
+			video->pitch = (video->pitch + 7) & ~7;
+			video->pixels = (void *)malloc(video->h*video->pitch);
 			if ( video->pixels == NULL ) {
 				if ( video != current ) {
 					SDL_FreeSurface(video);
@@ -1181,6 +1370,12 @@ SDL_Surface *DX5_SetVideoMode(_THIS, SDL_Surface *current,
 		}
 		dd_surface3 = NULL;
 		video->flags |= SDL_SWSURFACE;
+		if ( (flags & SDL_RESIZABLE) && !(flags & SDL_NOFRAME) ) {
+			video->flags |= SDL_RESIZABLE;
+		}
+		if ( flags & SDL_NOFRAME ) {
+			video->flags |= SDL_NOFRAME;
+		}
 	} else {
 		/* Necessary if we're going from window to fullscreen */
 		if ( video->pixels != NULL ) {
@@ -1220,6 +1415,7 @@ SDL_Surface *DX5_SetVideoMode(_THIS, SDL_Surface *current,
 	if ( (flags & SDL_FULLSCREEN) != SDL_FULLSCREEN ) {
 		RECT bounds;
 		int  x, y;
+		UINT swp_flags;
 
 		/* Create and set a clipper on our primary surface */
 		if ( SDL_clipper == NULL ) {
@@ -1266,8 +1462,11 @@ SDL_Surface *DX5_SetVideoMode(_THIS, SDL_Surface *current,
 		if ( y < 0 ) { /* Cover up title bar for more client area */
 			y -= GetSystemMetrics(SM_CYCAPTION)/2;
 		}
-		SetWindowPos(SDL_Window, NULL, x, y, width, height,
-					(SWP_NOCOPYBITS | SWP_NOZORDER));
+		swp_flags = (SWP_NOCOPYBITS | SWP_NOZORDER);
+		if ( was_visible ) {
+			swp_flags |= SWP_NOMOVE;
+		}
+		SetWindowPos(SDL_Window, NULL, x, y, width, height, swp_flags);
 		SDL_resizing = 0;
 	}
 	ShowWindow(SDL_Window, SW_SHOW);
@@ -1479,7 +1678,7 @@ static int DX5_AllocHWSurface(_THIS, SDL_Surface *surface)
 {
 	/* DDraw limitation -- you need to set cooperative level first */
 	if ( SDL_primary == NULL ) {
-		SDL_SetError("You must set the video mode first");
+		SDL_SetError("You must set a non-GL video mode first");
 		return(-1);
 	}
 	return(DX5_AllocDDSurface(this, surface, NULL, SDL_HWSURFACE));
@@ -1499,7 +1698,7 @@ void PrintSurface(char *title, LPDIRECTDRAWSURFACE3 surface, Uint32 flags)
 	IDirectDrawSurface3_Unlock(surface, NULL);
 	
 	fprintf(stderr, "%s:\n", title);
-	fprintf(stderr, "\tSize: %dx%d in %s at %d bpp (pitch = %d)\n",
+	fprintf(stderr, "\tSize: %dx%d in %s at %ld bpp (pitch = %ld)\n",
 		ddsd.dwWidth, ddsd.dwHeight,
 		(flags & SDL_HWSURFACE) ? "hardware" : "software",
 #if defined(NONAMELESSUNION)
@@ -1507,7 +1706,7 @@ void PrintSurface(char *title, LPDIRECTDRAWSURFACE3 surface, Uint32 flags)
 #else
 		ddsd.ddpfPixelFormat.dwRGBBitCount, ddsd.lPitch);
 #endif
-	fprintf(stderr, "\tR = 0x%p, G = 0x%p, B = 0x%p\n", 
+	fprintf(stderr, "\tR = 0x%X, G = 0x%X, B = 0x%X\n", 
 #if defined(NONAMELESSUNION)
 	     		ddsd.ddpfPixelFormat.u2.dwRBitMask,
 	     		ddsd.ddpfPixelFormat.u3.dwGBitMask,
@@ -1735,14 +1934,18 @@ static int DX5_FlipHWSurface(_THIS, SDL_Surface *surface)
 
 static void DX5_FreeHWSurface(_THIS, SDL_Surface *surface)
 {
-	if ( surface->hwdata->dd_surface != SDL_primary ) {
-		IDirectDrawSurface3_Release(surface->hwdata->dd_surface);
+	if ( surface->hwdata ) {
+		if ( surface->hwdata->dd_surface != SDL_primary ) {
+			IDirectDrawSurface3_Release(surface->hwdata->dd_surface);
+		}
+		free(surface->hwdata);
+		surface->hwdata = NULL;
 	}
-	free(surface->hwdata);
 }
 
 void DX5_WindowUpdate(_THIS, int numrects, SDL_Rect *rects)
 {
+	HRESULT result;
 	int i;
 	RECT src, dst;
 
@@ -1755,9 +1958,16 @@ void DX5_WindowUpdate(_THIS, int numrects, SDL_Rect *rects)
 		dst.left = SDL_bounds.left+src.left;
 		dst.bottom = SDL_bounds.top+src.bottom;
 		dst.right = SDL_bounds.left+src.right;
-		IDirectDrawSurface3_Blt(SDL_primary, &dst, 
+		result = IDirectDrawSurface3_Blt(SDL_primary, &dst, 
 					this->screen->hwdata->dd_surface, &src,
 							DDBLT_WAIT, NULL);
+		/* Doh!  Check for lost surface and restore it */
+		if ( result == DDERR_SURFACELOST ) {
+			IDirectDrawSurface3_Restore(SDL_primary);
+			IDirectDrawSurface3_Blt(SDL_primary, &dst, 
+					this->screen->hwdata->dd_surface, &src,
+							DDBLT_WAIT, NULL);
+		}
 	}
 }
 
@@ -1869,43 +2079,131 @@ static void DX5_CompressPalette(_THIS, SDL_Color *colors, int ncolors, int maxco
 }
 
 /* Set the system colormap in both fullscreen and windowed modes */
-int DX5_SetColors(_THIS, int firstcolor, int ncolors)
+int DX5_SetColors(_THIS, int firstcolor, int ncolors, SDL_Color *colors)
 {
-	SDL_Palette *palette;
 	int i;
 	int alloct_all;
 
 	/* Copy palette colors into display palette */
-	palette = this->screen->format->palette;
+	alloct_all = 0;
 	if ( SDL_palette != NULL ) {
 		if ( (this->screen->flags&SDL_FULLSCREEN) == SDL_FULLSCREEN ) {
 			/* We can set all entries explicitly */
-			for ( i=firstcolor; i<(firstcolor+ncolors); ++i ) {
-				SDL_colors[i].peRed = palette->colors[i].r;
-				SDL_colors[i].peGreen = palette->colors[i].g;
-				SDL_colors[i].peBlue = palette->colors[i].b;
+			for ( i=0; i< ncolors; ++i ) {
+			        int j = firstcolor + i;
+				SDL_colors[j].peRed = colors[i].r;
+				SDL_colors[j].peGreen = colors[i].g;
+				SDL_colors[j].peBlue = colors[i].b;
 			}
 			IDirectDrawPalette_SetEntries(SDL_palette, 0,
 				firstcolor, ncolors, &SDL_colors[firstcolor]);
 			alloct_all = 1;
 		} else {
 			/* Grab the 236 most diverse colors in the palette */
-			DX5_CompressPalette(this,
-			              palette->colors, palette->ncolors, 236);
+			DX5_CompressPalette(this, colors, ncolors, 236);
 			/* This sends an WM_PALETTECHANGED message to us */
 			colorchange_expected = 1;
 			IDirectDrawPalette_SetEntries(SDL_palette, 0,
 							0, 256, SDL_colors);
-			alloct_all = 0;
 		}
 	}
 	return(alloct_all);
 }
 
+static void DX5_SwapGamma(_THIS)
+{
+	return;
+}
+
+/* Gamma code is only available on DirectX 7 and newer */
+#ifdef IID_IDirectDrawGammaControl
+
+static int DX5_SetGammaRamp(_THIS, Uint16 *ramp)
+{
+	LPDIRECTDRAWGAMMACONTROL gamma;
+	DDGAMMARAMP gamma_ramp;
+	HRESULT result;
+
+	/* Check for a video mode! */
+	if ( ! SDL_primary ) {
+		SDL_SetError("A video mode must be set for gamma correction");
+		return(-1);
+	}
+
+	/* Get the gamma control object */
+	result = IDirectDrawSurface3_QueryInterface(SDL_primary,
+			&IID_IDirectDrawGammaControl, (LPVOID *)&gamma);
+	if ( result != DD_OK ) {
+		SetDDerror("DirectDrawSurface3::QueryInterface(GAMMA)", result);
+		return(-1);
+	}
+
+	/* Set up the gamma ramp */
+	memcpy(gamma_ramp.red, &ramp[0*256], 256*sizeof(*ramp));
+	memcpy(gamma_ramp.green, &ramp[1*256], 256*sizeof(*ramp));
+	memcpy(gamma_ramp.blue, &ramp[2*256], 256*sizeof(*ramp));
+	result = IDirectDrawGammaControl_SetGammaRamp(gamma, 0, &gamma_ramp);
+	if ( result != DD_OK ) {
+		SetDDerror("DirectDrawGammaControl::SetGammaRamp()", result);
+	}
+
+	/* Release the interface and return */
+	IDirectDrawGammaControl_Release(gamma);
+	return (result == DD_OK) ? 0 : -1;
+}
+
+static int DX5_GetGammaRamp(_THIS, Uint16 *ramp)
+{
+	LPDIRECTDRAWGAMMACONTROL gamma;
+	DDGAMMARAMP gamma_ramp;
+	HRESULT result;
+
+	/* Check for a video mode! */
+	if ( ! SDL_primary ) {
+		SDL_SetError("A video mode must be set for gamma correction");
+		return(-1);
+	}
+
+	/* Get the gamma control object */
+	result = IDirectDrawSurface3_QueryInterface(SDL_primary,
+			&IID_IDirectDrawGammaControl, (LPVOID *)&gamma);
+	if ( result != DD_OK ) {
+		SetDDerror("DirectDrawSurface3::QueryInterface(GAMMA)", result);
+		return(-1);
+	}
+
+	/* Set up the gamma ramp */
+	result = IDirectDrawGammaControl_GetGammaRamp(gamma, 0, &gamma_ramp);
+	if ( result == DD_OK ) {
+		memcpy(&ramp[0*256], gamma_ramp.red, 256*sizeof(*ramp));
+		memcpy(&ramp[1*256], gamma_ramp.green, 256*sizeof(*ramp));
+		memcpy(&ramp[2*256], gamma_ramp.blue, 256*sizeof(*ramp));
+	} else {
+		SetDDerror("DirectDrawGammaControl::GetGammaRamp()", result);
+	}
+
+	/* Release the interface and return */
+	IDirectDrawGammaControl_Release(gamma);
+	return (result == DD_OK) ? 0 : -1;
+}
+
+#endif /* IID_IDirectDrawGammaControl */
+
 void DX5_VideoQuit(_THIS)
 {
 	int i, j;
 
+	/* If we're fullscreen GL, we need to reset the display */
+	if ( this->screen != NULL ) {
+		if ( (this->screen->flags & (SDL_OPENGL|SDL_FULLSCREEN)) ==
+		                            (SDL_OPENGL|SDL_FULLSCREEN) ) {
+			ChangeDisplaySettings(NULL, 0);
+		}
+		if ( this->screen->flags & SDL_OPENGL ) {
+			WIN_GL_ShutDown(this);
+		}
+	}
+
 	/* Free any palettes we used */
 	if ( SDL_palette != NULL ) {
 		IDirectDrawPalette_Release(SDL_palette);
@@ -1972,7 +2270,7 @@ static void DX5_Recolor8Bit(_THIS, SDL_Surface *surface, Uint8 *mapping)
 void DX5_PaletteChanged(_THIS, HWND window)
 {
 	SDL_Palette *palette;
-	SDL_Color *saved;
+	SDL_Color *saved = NULL;
 	HDC hdc;
 	int i;
 	PALETTEENTRY *entries;
@@ -1982,7 +2280,9 @@ void DX5_PaletteChanged(_THIS, HWND window)
 		return;
 
 	/* We need to get the colors as they were set */
-	palette = SDL_VideoSurface->format->palette;
+	palette = this->physpal;
+	if(!palette)
+	        palette = SDL_VideoSurface->format->palette;
 	if ( palette == NULL ) { /* Sometimes we don't have a palette */
 		return;
 	}
