@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_x11image_c.h,v 1.1.2.3 2000/03/20 15:02:48 hercules Exp $";
+ "@(#) $Id: SDL_x11image_c.h,v 1.1.2.5 2001/02/10 07:20:15 hercules Exp $";
 #endif
 
 #include "SDL_x11video.h"
@@ -37,4 +37,6 @@ extern int X11_LockHWSurface(_THIS, SDL_Surface *surface);
 extern void X11_UnlockHWSurface(_THIS, SDL_Surface *surface);
 extern int X11_FlipHWSurface(_THIS, SDL_Surface *surface);
 
+extern void X11_DisableAutoRefresh(_THIS);
+extern void X11_EnableAutoRefresh(_THIS);
 extern void X11_RefreshDisplay(_THIS);
