@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_x11mouse_c.h,v 1.2 2000/01/23 02:04:36 hercules Exp $";
+ "@(#) $Id: SDL_x11mouse_c.h,v 1.1.2.5 2001/02/10 07:20:15 hercules Exp $";
 #endif
 
 #include "SDL_x11video.h"
@@ -31,7 +31,7 @@ static char rcsid =
 extern void X11_FreeWMCursor(_THIS, WMcursor *cursor);
 extern WMcursor *X11_CreateWMCursor(_THIS,
 		Uint8 *data, Uint8 *mask, int w, int h, int hot_x, int hot_y);
-extern Cursor X11_GetWMXCursor(WMcursor *cursor);
 extern int X11_ShowWMCursor(_THIS, WMcursor *cursor);
 extern void X11_WarpWMCursor(_THIS, Uint16 x, Uint16 y);
+extern void X11_CheckMouseModeNoLock(_THIS);
 extern void X11_CheckMouseMode(_THIS);
