@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_x11wm.c,v 1.7 2000/02/22 23:05:47 hercules Exp $";
+ "@(#) $Id: SDL_x11wm.c,v 1.2.2.14 2001/03/21 18:39:51 hercules Exp $";
 #endif
 
 #include <stdlib.h>
@@ -32,6 +32,7 @@ static char rcsid =
 
 #include "SDL_version.h"
 #include "SDL_error.h"
+#include "SDL_timer.h"
 #include "SDL_video.h"
 #include "SDL_syswm.h"
 #include "SDL_events_c.h"
@@ -54,27 +55,43 @@ void X11_SetIcon(_THIS, SDL_Surface *icon, Uint8 *mask)
 #endif
 	GC GC;
 	XGCValues GCvalues;
-	int i, b;
+	int i, b, dbpp;
 	SDL_Rect bounds;
 	Uint8 *LSBmask, *color_tried;
+	Visual *dvis;
 
 	/* Lock the event thread, in multi-threading environments */
 	SDL_Lock_EventThread();
 
-	/* Create an image in the screen format */
+	/* The icon must use the default visual, depth and colormap of the
+	   screen, so it might need a conversion */
+	dbpp = DefaultDepth(SDL_Display, SDL_Screen);
+	switch(dbpp) {
+	case 15:
+	    dbpp = 16; break;
+	case 24:
+	    dbpp = 32; break;
+	}
+	dvis = DefaultVisual(SDL_Display, SDL_Screen);
+
+	/* The Visual struct is supposed to be opaque but we cheat a little */
 	sicon = SDL_CreateRGBSurface(SDL_SWSURFACE, icon->w, icon->h,
-					this->screen->format->BitsPerPixel,
-						this->screen->format->Rmask,
-						this->screen->format->Gmask,
-						this->screen->format->Bmask, 0);
+				     dbpp,
+				     dvis->red_mask, dvis->green_mask,
+				     dvis->blue_mask, 0);
+
 	if ( sicon == NULL ) {
 		goto done;
 	}
-	if ( this->screen->format->palette ) {
-		memcpy(sicon->format->palette->colors,
-		     this->screen->format->palette->colors,
-		     this->screen->format->palette->ncolors*sizeof(SDL_Color));
+	/* If we already have allocated colours from the default colormap,
+	   copy them */
+	if(SDL_Visual == dvis && SDL_XColorMap == SDL_DisplayColormap
+	   && this->screen->format->palette && sicon->format->palette) {
+	    memcpy(sicon->format->palette->colors,
+		   this->screen->format->palette->colors,
+		   this->screen->format->palette->ncolors * sizeof(SDL_Color));
 	}
+
 	bounds.x = 0;
 	bounds.y = 0;
 	bounds.w = icon->w;
@@ -90,20 +107,22 @@ void X11_SetIcon(_THIS, SDL_Surface *icon, Uint8 *mask)
 		XColor wanted;
 
 		palette = sicon->format->palette;
-		color_tried = (Uint8 *)malloc(palette->ncolors);
+		color_tried = malloc(palette->ncolors);
 		if ( color_tried == NULL ) {
 			goto done;
 		}
 		if ( SDL_iconcolors != NULL ) {
 			free(SDL_iconcolors);
 		}
-		SDL_iconcolors = (Uint8 *)malloc(palette->ncolors);
+		SDL_iconcolors = malloc(palette->ncolors
+					* sizeof(*SDL_iconcolors));
 		if ( SDL_iconcolors == NULL ) {
 			free(color_tried);
 			goto done;
 		}
 		memset(color_tried, 0, palette->ncolors);
-		memset(SDL_iconcolors, 0, palette->ncolors);
+		memset(SDL_iconcolors, 0,
+		       palette->ncolors * sizeof(*SDL_iconcolors));
 
 		p = (Uint8 *)sicon->pixels; 
 		for ( i = sicon->w*sicon->h; i > 0; --i, ++p ) {
@@ -137,7 +156,7 @@ void X11_SetIcon(_THIS, SDL_Surface *icon, Uint8 *mask)
 			LSBmask[i] |= (((mask[i]>>b)&0x01)<<(7-b));
 	}
 	mask_pixmap = XCreatePixmapFromBitmapData(SDL_Display, WMwindow,
-					LSBmask, sicon->w, sicon->h, 1L, 0L, 1);
+		          (char *)LSBmask, sicon->w, sicon->h, 1L, 0L, 1);
 
 	/* Transfer the image to an X11 pixmap */
 	icon_image = XCreateImage(SDL_Display,
@@ -227,6 +246,9 @@ SDL_GrabMode X11_GrabInputNoLock(_THIS, SDL_GrabMode mode)
 	if ( this->screen == NULL ) {
 		return(SDL_GRAB_OFF);
 	}
+	if ( ! SDL_Window ) {
+		return(mode);	/* Will be set later on mode switch */
+	}
 	if ( mode == SDL_GRAB_OFF ) {
 		XUngrabPointer(SDL_Display, CurrentTime);
 		if ( this->screen->flags & SDL_FULLSCREEN ) {
@@ -266,6 +288,10 @@ SDL_GrabMode X11_GrabInputNoLock(_THIS, SDL_GrabMode mode)
 #endif
 		XGrabKeyboard(SDL_Display, WMwindow, True,
 			GrabModeAsync, GrabModeAsync, CurrentTime);
+
+		/* Raise the window if we grab the mouse */
+		if ( !(this->screen->flags & SDL_FULLSCREEN) )
+			XRaiseWindow(SDL_Display, WMwindow);
 	}
 	XSync(SDL_Display, False);
 
@@ -300,7 +326,7 @@ int X11_GetWMInfo(_THIS, SDL_SysWMinfo *info)
 	if ( info->version.major <= SDL_MAJOR_VERSION ) {
 		info->subsystem = SDL_SYSWM_X11;
 		info->info.x11.display = SDL_Display;
-		info->info.x11.window = WMwindow;
+		info->info.x11.window = SDL_Window;
 		if ( SDL_VERSIONNUM(info->version.major,
 		                    info->version.minor,
 		                    info->version.patch) >= 1002 ) {
